import os
#os.environ['THEANO_FLAGS'] = "device=gpu0"
import sys


from sklearn.model_selection import StratifiedKFold
from tensorflow.python.keras.callbacks import ModelCheckpoint, EarlyStopping
from tensorflow.python.keras.layers import Dropout, Dense, Flatten, Activation, Conv1D, LeakyReLU, Bidirectional, LSTM, \
    BatchNormalization
from tensorflow.python.keras.models import Sequential, load_model
# from tensorflow.python.keras.optimizers import Adam
from tensorflow.keras.optimizers import Adam
from tensorflow.python.keras.optimizer_v2.gradient_descent import SGD

from config.config import Config
from data.data import get_single_data

from tools.tools import score2, split_seq_df, get_label

sys.setrecursionlimit(15000)
import numpy as np
import h5py






def seq_to01_to0123(seq):

#     seq, label = read_seq_label(filename)

    nrows = len(seq)
    seq_len = len(seq[0])


    seq_01 = np.zeros((nrows, seq_len, 4), dtype='int')
    # seq_0123 = np.zeros((nrows, seq_len), dtype='int')

    for i in range(nrows):
        one_seq = seq[i]
        one_seq = one_seq.replace('A', '0')
        one_seq = one_seq.replace('C', '1')
        one_seq = one_seq.replace('G', '2')
        one_seq = one_seq.replace('U', '3')
        one_seq = one_seq.replace('T', '3')
        seq_start = 0

        for j in range(seq_len):
            # seq_0123[i, j] = int(one_seq[j - seq_start])
            if j < seq_start:
                seq_01[i, j, :] = 0.25
            else:
                try:
                    seq_01[i, j, int(one_seq[j - seq_start])] = 1
                except:
                    seq_01[i, j, :] = 0.25
    return seq_01, None

# Value             |Best Value So Far |Hyperparameter
# 150               |100               |filters_1
# 10                |10                |kernel_size_1
# 250               |250               |filters_2
# 2                 |2                 |kernel_size_2
# 100               |250               |filters_3
# 2                 |2                 |kernel_size_3
# 100               |250               |filters_5
# 10                |8                 |kernel_size_5
# 320               |320               |dense_1
# 0.01              |0.01              |learning_rate

def create_model(config):
    print('building model...............')
    model = Sequential()
    # params = {
    #     "filters_1": 250,
    #     "kernel_size_1": 8,
    #     "filters_2": 250,
    #     "kernel_size_2": 10,
    #     "filters_3": 150,
    #     "kernel_size_3": 2,
    #     "filters_4": 100,
    #     "kernel_size_4": 3,
    #     "filters_5": 100,
    #     "kernel_size_5": 9,
    #     "dense_1": 64
    # }
    params = {
        "filters_1": 250,
        "kernel_size_1": 7,
        "filters_2": 250,
        "kernel_size_2": 10,
        "filters_3": 250,
        "kernel_size_3": 10,
        "filters_4": 100,
        "kernel_size_4": 2,
        "filters_5": 150,
        "kernel_size_5": 10,
        "dense_1": 1024,
        "dense_2": 320
    }
    #************************* convolutional layer  ***********************
    num_filters = config.num_filters

    model.add(Conv1D(
        # input_dim=4,
                            # input_length=X_train.shape[1],
                            # input_shape=(101,4),
                            input_shape=(201,4),
                            filters=params["filters_1"],
                            kernel_size=params["kernel_size_1"],
                            padding="same",
                            activation="linear",
                            strides=1,
                            #W_regularizer = l2(0.01),
                            kernel_initializer='he_normal',
                            name="cov1"))
    model.add(LeakyReLU(alpha=.001))
    model.add(Dropout(0.2))


    model.add(Conv1D(filters=params["filters_2"],
                            kernel_size=params["kernel_size_2"],
                            padding="same",
                            activation="linear",
                            strides=1, kernel_initializer='he_normal',
                            name = "cov2"))
    model.add(LeakyReLU(alpha=.001))
    model.add(Dropout(0.2))



    model.add(Conv1D(filters=params["filters_3"],
                            kernel_size=params["kernel_size_3"],
                            padding="same",
                            activation="linear",
                            strides=1, kernel_initializer='he_normal',
                            name="cov3"))
    model.add(LeakyReLU(alpha=.001))
    model.add(Dropout(0.2))


    model.add(Conv1D(filters=params["filters_4"],
                            kernel_size=params["kernel_size_4"],
                            padding="same",
                            activation="linear",
                            strides=1, kernel_initializer='he_normal',
                            name = "cov4"))
    model.add(LeakyReLU(alpha=.001))
    model.add(Dropout(0.2))



    model.add(Conv1D(filters=params["filters_5"],
                            kernel_size=params["kernel_size_5"],
                            padding="same",
                            activation="linear",
                            strides=1,
                            kernel_initializer='he_normal',
                            name = "cov5"))
    model.add(LeakyReLU(alpha=.001))
    model.add(Dropout(0.5))


    model.add(Flatten())
    model.add(Dense(units=params["dense_1"], kernel_initializer='he_normal'))
    model.add(LeakyReLU(alpha=.001))
    model.add(Dropout(0.5))
    model.add(Dense(units=params["dense_2"], kernel_initializer='he_normal'))
    model.add(LeakyReLU(alpha=.001))
    model.add(Dropout(0.5))
    model.add(Dense(units=1))
    model.add(Activation('sigmoid'))
    # print(model.summary())
    return model

class TA_keras():
    def __init__(self,config):
        idx = 1
        self.config = config
        self.best_model = None
        self.temp_model = None
        self.global_model_save_path = config.global_model_save_path
        self.bestmodel_path = config.model_save_path+config.model_name+"_.h5"
        print("model path:",self.bestmodel_path)


    def fit(self,X,y,X_train=None,X_dev=None):
        if X_train is None:
            X_train, X_dev = split_seq_df(X)
        y_train = get_label(X_train)
        y_dev = get_label(X_dev)
        X_train,_ = seq_to01_to0123(X_train.iloc[:,1].values)
        X_dev,_ = seq_to01_to0123(X_dev.iloc[:,1].values)

        #--------------------------------------
        # X_train = X_train.astype(np.float)
        # X_dev = X_dev.astype(np.float)
        #---------------------------------------

        if self.config.load_global_pretrain_model:
            print("loading global pretrain model ...")
            model = load_model(self.config.global_model_save_path)
        else:
            model = create_model(self.config)

        sgd = SGD(lr=self.config.learning_rate, momentum=0.9, decay=1e-6, nesterov=True)
        checkpointer = ModelCheckpoint(filepath=self.bestmodel_path, verbose=0, save_best_only=True)
        earlystopper = EarlyStopping(monitor='val_loss', patience=self.config.patience, verbose=0)
        model.compile(loss='binary_crossentropy', optimizer=sgd, metrics=['accuracy'])
        model.fit(X_train,
                  y_train,
                  batch_size=self.config.batch_size,
                  epochs=self.config.num_epochs,
                  shuffle=True,
                  verbose=0,
                  validation_data=(X_dev, y_dev),
                  callbacks=[checkpointer, earlystopper])
        # self.best_model = load_model(self.bestmodel_path,custom_objects={'Attention': Attention})
        self.best_model = load_model(self.bestmodel_path)
        print('training done!')

    def predict_proba(self,X):
        # model = load_model(self.bestmodel_path)
        #test
        model = self.best_model
        X, _ = seq_to01_to0123(X.iloc[:, 1].values)
        #---------------------------
        # X = X.astype(np.float)
        #-------------------------------
        pred_prob_test = model.predict(X, verbose=0)
        pred_prob_test = np.squeeze(pred_prob_test)
        # print(score2(y,pred_prob_test))
        return pred_prob_test

    # def predict(self,X,y):
    #     f = h5py.File(self.bestmodel_path, 'r+')
    #     del f['optimizer_weights']
    #     f.close()
    #     model = load_model(self.bestmodel_path)
    #     pred_test = model.predict_classes(X, verbose=0)
    #     pred_test = np.squeeze(pred_test)
    #     # print(score2(y,pred_prob_test))
    #     return pred_test


